@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,7 +23,8 @@ the scope, to make sure it can be used without.
 
 Note: This is using "optparse", because "argparse" is only Python 2.7 and
 higher, and we still support Python 2.6 due to the RHELs still being used,
-and despite the long deprecation, it's in every later release.
+and despite the long deprecation, it's in every later release, and actually
+pretty good.
 """
 
 import os
@@ -34,7 +35,10 @@ from optparse import SUPPRESS_HELP, OptionGroup, OptionParser
 from nuitka.PythonFlavors import (
     isAnacondaPython,
     isApplePython,
+    isCPythonOfficialPackage,
     isDebianPackagePython,
+    isFedoraPackagePython,
+    isHomebrewPython,
     isMSYS2MingwPython,
     isNuitkaPython,
     isPyenvPython,
@@ -48,6 +52,7 @@ from nuitka.utils.Utils import (
     getWindowsRelease,
     isLinux,
     isPosixWindows,
+    withNoSyntaxWarning,
 )
 from nuitka.Version import getCommercialVersion, getNuitkaVersion
 
@@ -72,6 +77,10 @@ def _getPythonFlavor():
         return "WinPython"
     elif isDebianPackagePython():
         return "Debian Python"
+    elif isFedoraPackagePython():
+        return "Fedora Python"
+    elif isHomebrewPython():
+        return "Homebrew Python"
     elif isApplePython():
         return "Apple Python"
     elif isPyenvPython():
@@ -80,6 +89,8 @@ def _getPythonFlavor():
         return "MSYS2 Posix"
     elif isMSYS2MingwPython():
         return "MSYS2 MinGW"
+    elif isCPythonOfficialPackage():
+        return "CPython Official"
     else:
         return "Unknown"
 
@@ -96,22 +107,47 @@ def _getVersionInformationValues():
     yield "Arch: %s" % getArchitecture()
 
     if isLinux():
-        yield "Distribution: %s %s" % getLinuxDistribution()
+        dist_name, dist_base, dist_version = getLinuxDistribution()
+
+        if dist_base is not None:
+            yield "Distribution: %s (based on %s) %s" % (
+                dist_name,
+                dist_base,
+                dist_version,
+            )
+        else:
+            yield "Distribution: %s %s" % (dist_name, dist_version)
 
     if getOS() == "Windows":
         yield "WindowsRelease: %s" % getWindowsRelease()
 
 
-parser = OptionParser(
+class OurOptionParser(OptionParser):
+    # spell-checker: ignore rargs
+    def _process_long_opt(self, rargs, values):
+        arg = rargs[0]
+
+        if "=" not in arg:
+            opt = self._match_long_opt(arg)
+            option = self._long_opt[opt]
+            if option.takes_value():
+                self.error(
+                    "The '%s' option requires an argument with '%s='." % (opt, opt)
+                )
+
+        return OptionParser._process_long_opt(self, rargs, values)
+
+
+parser = OurOptionParser(
     usage=usage,
     version="\n".join(_getVersionInformationValues()),
 )
 
 parser.add_option(
     "--module",
-    action="store_false",
-    dest="executable",
-    default=True,
+    action="store_true",
+    dest="module_mode",
+    default=False,
     help="""\
 Create an extension module executable instead of a program. Defaults to off.""",
 )
@@ -124,9 +160,8 @@ parser.add_option(
     help="""\
 Enable standalone mode for output. This allows you to transfer the created binary
 to other machines without it using an existing Python installation. This also
-means it will become big. It implies these option: "--follow-imports". You may also
-want to use "--python-flag=no_site" to avoid the "site.py" module, which can save
-a lot of code dependencies. Defaults to off.""",
+means it will become big. It implies these option: "--follow-imports" and
+"--python-flag=no_site". Defaults to off.""",
 )
 
 parser.add_option(
@@ -156,20 +191,6 @@ parser.add_option(
     help=SUPPRESS_HELP,
 )
 
-
-if os.name == "nt":
-    parser.add_option(
-        "--python-arch",
-        action="store",
-        dest="python_arch",
-        choices=("x86", "x86_64"),
-        default=None,
-        help="""\
-Architecture of Python to use. One of "x86" or "x86_64".
-Defaults to what you run Nuitka with (currently "%s")."""
-        % (getArchitecture()),
-    )
-
 parser.add_option(
     "--python-debug",
     action="store_true",
@@ -192,7 +213,7 @@ enforces a specific mode. These are options that also exist to standard
 Python executable. Currently supported: "-S" (alias "no_site"),
 "static_hashes" (do not use hash randomization), "no_warnings" (do not
 give Python runtime warnings), "-O" (alias "no_asserts"), "no_docstrings"
-(do not use docstrings), and "-m".  Default empty.""",
+(do not use doc strings), "-u" (alias "unbuffered") and "-m".  Default empty.""",
 )
 
 parser.add_option(
@@ -203,11 +224,15 @@ parser.add_option(
     default=None,
     help="""\
 If using Python3.3 or Python3.4, provide the path of a Python binary to use
-for Scons. Otherwise Nuitka can use what you run Nuitka with or a "scons"
-binary that is found in PATH, or a Python installation from Windows registry.""",
+for Scons. Otherwise Nuitka can use what you run Nuitka with or a Python
+installation from Windows registry. On Windows Python 3.5 or higher is
+needed. On non-Windows, Python 2.6 or 2.7 will do as well.""",
 )
 
-parser.add_option(
+warnings_group = OptionGroup(parser, "Control the warnings to be given by Nuitka")
+
+
+warnings_group.add_option(
     "--warn-implicit-exceptions",
     action="store_true",
     dest="warn_implicit_exceptions",
@@ -216,7 +241,7 @@ parser.add_option(
 Enable warnings for implicit exceptions detected at compile time.""",
 )
 
-parser.add_option(
+warnings_group.add_option(
     "--warn-unusual-code",
     action="store_true",
     dest="warn_unusual_code",
@@ -225,7 +250,7 @@ parser.add_option(
 Enable warnings for unusual code detected at compile time.""",
 )
 
-parser.add_option(
+warnings_group.add_option(
     "--assume-yes-for-downloads",
     action="store_true",
     dest="assume_yes_for_downloads",
@@ -237,8 +262,23 @@ from nul device, e.g. "</dev/null" or "<NUL:". Default is to prompt.""",
 )
 
 
+warnings_group.add_option(
+    "--nowarn-mnemonic",
+    action="append",
+    dest="nowarn_mnemonics",
+    metavar="MNEMONIC",
+    default=[],
+    help="""\
+Disable warning for a given mnemonic. These are given to make sure you are aware of
+certain topics, and typically point to the Nuitka website. The mnemonic is the part
+of the URL at the end, without the HTML suffix. Can be given multiple times and
+accepts shell pattern. Default empty.""",
+)
+
+parser.add_option_group(warnings_group)
+
 include_group = OptionGroup(
-    parser, "Control the inclusion of modules and packages in result."
+    parser, "Control the inclusion of modules and packages in result"
 )
 
 include_group.add_option(
@@ -274,9 +314,12 @@ include_group.add_option(
     metavar="MODULE/PACKAGE",
     default=[],
     help="""\
-Include the content of that directory, no matter if it's used by the given main
-program in a visible form. Overrides all other inclusion options. Can be given
-multiple times. Default empty.""",
+Include also the code found in that directory, considering as if
+they are each given as a main file. Overrides all other inclusion
+options. You ought to prefer other inclusion options, that go by
+names, rather than filenames, as this always includes too much,
+and find things through being in "sys.path". Can be given multiple
+times. Default empty.""",
 )
 
 include_group.add_option(
@@ -372,7 +415,7 @@ times. Default empty.""",
 parser.add_option_group(follow_group)
 
 
-data_group = OptionGroup(parser, "Data files for standalone/onefile mode")
+data_group = OptionGroup(parser, "Data files")
 
 data_group.add_option(
     "--include-package-data",
@@ -388,18 +431,18 @@ empty.""",
 )
 
 data_group.add_option(
-    "--include-data-file",
+    "--include-data-files",
     action="append",
     dest="data_files",
     metavar="DESC",
     default=[],
     help="""\
 Include data files by filenames in the distribution. There are many
-allowed forms. With '--include-data-file=/path/to/file/*.txt=folder_name/some.txt' it
+allowed forms. With '--include-data-files=/path/to/file/*.txt=folder_name/some.txt' it
 will copy a single file and complain if it's multiple. With
-'--include-data-file=/path/to/files/*.txt=folder_name/' it will put
+'--include-data-files=/path/to/files/*.txt=folder_name/' it will put
 all matching files into that folder. For recursive copy there is a
-form with 3 values that '--include-data-file=/path/to/scan=folder_name=**/*.txt'
+form with 3 values that '--include-data-files=/path/to/scan=folder_name=**/*.txt'
 that will preserve directory structure. Default empty.""",
 )
 
@@ -411,35 +454,46 @@ data_group.add_option(
     default=[],
     help="""\
 Include data files from complete directory in the distribution. This is
-recursive. Check '--include-data-file' with patterns if you want non-recursive
-inclusion. An example would be '--include-data-dir=/path/somedir=data/somedir'
+recursive. Check '--include-data-files' with patterns if you want non-recursive
+inclusion. An example would be '--include-data-dir=/path/some_dir=data/some_dir'
 for plain copy, of the whole directory. All files are copied, if you want to
-exclude files you need to remove them beforehand. Default empty.""",
+exclude files you need to remove them beforehand, or use '--noinclude-data-files'
+option to remove them. Default empty.""",
 )
 
-data_files_tags = [("inhibit", "do not include the file")]
-
-# TODO: Expose this when finished, pylint: disable=using-constant-test
-if False:
-    data_group.add_option(
-        "--data-file-tags",
-        action="append",
-        dest="data_tags",
-        metavar="DATA_TAGS",
-        default=[],
-        help="""\
-    For included data files, special handlings can be chosen. With the
-    commercial plugins, e.g. files can be included directly in the
-    binary. The list is completed by some plugins. With the current
-    list of plugins, these are available: %s.
-    The default is empty."""
-        % ",".join("'%s' (%s)" % d for d in data_files_tags),
-    )
-
+data_group.add_option(
+    "--noinclude-data-files",
+    action="append",
+    dest="data_files_inhibited",
+    metavar="PATTERN",
+    default=[],
+    help="""\
+Do not include data files matching the filename pattern given. This is against
+the target filename, not source paths. So ignore file pattern from package
+data for "package_name" should be matched as "package_name/*.txt". Default
+empty.""",
+)
 
 parser.add_option_group(data_group)
 
 
+dll_group = OptionGroup(parser, "DLL files")
+
+dll_group.add_option(
+    "--noinclude-dlls",
+    action="append",
+    dest="dll_files_inhibited",
+    metavar="PATTERN",
+    default=[],
+    help="""\
+Do not include DLL files matching the filename pattern given. This is against
+the target filename, not source paths. So ignore a DLL "someDLL" contained in
+the package "package_name" it should be matched as "package_name/someDLL.*".
+Default empty.""",
+)
+
+parser.add_option_group(dll_group)
+
 execute_group = OptionGroup(parser, "Immediate execution after compilation")
 
 execute_group.add_option(
@@ -470,9 +524,9 @@ execute_group.add_option(
     dest="keep_pythonpath",
     default=False,
     help="""\
-When immediately executing the created binary (--execute), don't reset
-PYTHONPATH. When all modules are successfully included, you ought to not need
-PYTHONPATH anymore.""",
+When immediately executing the created binary ('--execute'), don't reset
+'PYTHONPATH' environment. When all modules are successfully included, you
+ought to not need PYTHONPATH anymore.""",
 )
 
 parser.add_option_group(execute_group)
@@ -491,9 +545,32 @@ dump_group.add_option(
 parser.add_option_group(dump_group)
 
 
-codegen_group = OptionGroup(parser, "Code generation choices")
+compilation_group = OptionGroup(parser, "Compilation choices")
 
-codegen_group.add_option(
+compilation_group.add_option(
+    "--user-package-configuration-file",
+    action="append",
+    dest="user_yaml_files",
+    default=[],
+    metavar="USER_YAML",
+    help="""\
+User provided Yaml file with package configuration. You can include DLLs,
+remove bloat, add hidden dependencies. Check User Manual for a complete
+description of the format to use. Can be given multiple times. Defaults
+to empty.""",
+)
+
+compilation_group.add_option(
+    "--disable-bytecode-cache",
+    action="store_true",
+    dest="disable_bytecode_cache",
+    default=False,
+    help="""\
+Do not reuse dependency analysis results for modules, esp. from standard library,
+that are included as bytecode.""",
+)
+
+compilation_group.add_option(
     "--full-compat",
     action="store_false",
     dest="improved",
@@ -502,11 +579,11 @@ codegen_group.add_option(
 Enforce absolute compatibility with CPython. Do not even allow minor
 deviations from CPython behavior, e.g. not having better tracebacks
 or exception messages which are not really incompatible, but only
-different. This is intended for tests only and should not be used
-for normal use.""",
+different or worse. This is intended for tests only and should *not*
+be used.""",
 )
 
-codegen_group.add_option(
+compilation_group.add_option(
     "--file-reference-choice",
     action="store",
     dest="file_reference_mode",
@@ -525,7 +602,23 @@ compatibility reasons, the "__file__" value will always have ".py" suffix
 independent of what it really is.""",
 )
 
-parser.add_option_group(codegen_group)
+compilation_group.add_option(
+    "--module-name-choice",
+    action="store",
+    dest="module_name_mode",
+    metavar="MODE",
+    choices=("original", "runtime"),
+    default=None,
+    help="""\
+Select what value "__name__" and "__package__" are going to be. With "runtime"
+(default for module mode), the created module uses the parent package to
+deduce the value of "__package__", to be fully compatible. The value "original"
+(default for other modes) allows for more static optimization to happen, but
+is incompatible for modules that normally can be loaded into any package.""",
+)
+
+
+parser.add_option_group(compilation_group)
 
 output_group = OptionGroup(parser, "Output choices")
 
@@ -595,9 +688,9 @@ production. Defaults to off.""",
 )
 
 debug_group.add_option(
-    "--unstripped",
+    "--unstriped",
     action="store_true",
-    dest="unstripped",
+    dest="unstriped",
     default=False,
     help="""\
 Keep debug info in the resulting object file for better debugger interaction.
@@ -616,7 +709,7 @@ Enable vmprof based profiling of time spent. Not working currently. Defaults to
 debug_group.add_option(
     "--internal-graph",
     action="store_true",
-    dest="graph",
+    dest="internal_graph",
     default=False,
     help="""\
 Create graph of optimization process internals, do not use for whole programs, but only
@@ -755,11 +848,11 @@ c_compiler_group.add_option(
     default=None,
     help="""\
 Enforce the use of specific MSVC version on Windows. Allowed values
-are e.g. "14.2" (MSVC 2019), specify an illegal value for a list of
-installed compilers, or use "latest". Notice that only latest
-MSVC is really supported, and you can use "latest" to enforce that.
+are e.g. "14.3" (MSVC 2022) and other MSVC version numbers, specify
+"list" for a list of installed compilers, or use "latest".
 
-Defaults to MSVC on Windows being used if installed, otherwise MinGW64.""",
+Defaults to latest MSVC being used if installed, otherwise MinGW64
+is used.""",
 )
 
 c_compiler_group.add_option(
@@ -908,8 +1001,7 @@ tracing_group.add_option(
     action="store_false",
     dest="progress_bar",
     default=True,
-    help="""Disable progress bar outputs (if tqdm is installed).
-Defaults to off.""",
+    help="""Disable progress bar. Defaults to off.""",
 )
 
 
@@ -949,7 +1041,7 @@ tracing_group.add_option(
     dest="compilation_report_filename",
     default=None,
     help="""\
-Report module inclusion in an XML output file. Default is off.""",
+Report module, data file, compilation details in an XML output file. Default is off.""",
 )
 
 tracing_group.add_option(
@@ -974,6 +1066,60 @@ Where to output --verbose, should be a filename. Default is standard output.""",
 
 parser.add_option_group(tracing_group)
 
+os_group = OptionGroup(parser, "General OS controls")
+
+os_group.add_option(
+    "--disable-console",
+    "--macos-disable-console",
+    "--windows-disable-console",
+    action="store_true",
+    dest="disable_console",
+    default=None,
+    help="""\
+When compiling for Windows or macOS, disable the console window and create a GUI
+application. Defaults to off.""",
+)
+
+os_group.add_option(
+    "--enable-console",
+    action="store_false",
+    dest="disable_console",
+    default=None,
+    help="""\
+When compiling for Windows or macOS, enable the console window and create a console
+application. This disables hints from certain modules, e.g. "PySide" that suggest
+to disable it. Defaults to true.""",
+)
+
+os_group.add_option(
+    "--force-stdout-spec",
+    "--windows-force-stdout-spec",
+    action="store",
+    dest="force_stdout_spec",
+    metavar="FORCE_STDOUT_SPEC",
+    default=None,
+    help="""\
+Force standard output of the program to go to this location. Useful for programs with
+disabled console and programs using the Windows Services Plugin of Nuitka commercial.
+Defaults to not active, use e.g. '%PROGRAM%.out.txt', i.e. file near your program.""",
+)
+
+os_group.add_option(
+    "--force-stderr-spec",
+    "--windows-force-stderr-spec",
+    action="store",
+    dest="force_stderr_spec",
+    metavar="FORCE_STDERR_SPEC",
+    default=None,
+    help="""\
+Force standard error of the program to go to this location. Useful for programs with
+disabled console and programs using the Windows Services Plugin of Nuitka commercial.
+Defaults to not active, use e.g. '%PROGRAM%.err.txt', i.e. file near your program.""",
+)
+
+
+parser.add_option_group(os_group)
+
 windows_group = OptionGroup(parser, "Windows specific controls")
 
 windows_group.add_option(
@@ -984,15 +1130,6 @@ windows_group.add_option(
     help=SUPPRESS_HELP,
 )
 
-windows_group.add_option(
-    "--windows-disable-console",
-    action="store_true",
-    dest="disable_console",
-    default=False,
-    help="""\
-When compiling for Windows, disable the console window. Defaults to off.""",
-)
-
 windows_group.add_option(
     "--windows-icon-from-ico",
     action="append",
@@ -1034,7 +1171,7 @@ windows_group.add_option(
 )
 
 windows_group.add_option(
-    "--windows-uac-uiaccess",
+    "--windows-uac-uiaccess",  # spell-checker: ignore uiaccess
     action="store_true",
     dest="windows_uac_uiaccess",
     metavar="WINDOWS_UAC_UIACCESS",
@@ -1119,7 +1256,7 @@ windows_group.add_option(
 )
 
 windows_group.add_option(
-    "--windows-onefile-tempdir-spec",
+    "--onefile-tempdir-spec",
     action="store",
     dest="onefile_tempdir_spec",
     metavar="ONEFILE_TEMPDIR_SPEC",
@@ -1128,52 +1265,21 @@ windows_group.add_option(
 Use this as a temporary folder. Defaults to '%TEMP%\\onefile_%PID%_%TIME%', i.e. system temporary directory.""",
 )
 
-windows_group.add_option(
-    "--windows-force-stdout-spec",
-    action="store",
-    dest="force_stdout_spec",
-    metavar="WINDOWS_FORCE_STDOUT_SPEC",
-    default=None,
-    help="""\
-Force standard output of the program to go to this location. Useful for programs with
-disabled console and programs using the Windows Services Plugin of Nuitka. Defaults
-to not active, use e.g. '%PROGRAM%.out.txt', i.e. file near your program.""",
-)
-
-windows_group.add_option(
-    "--windows-force-stderr-spec",
-    action="store",
-    dest="force_stderr_spec",
-    metavar="WINDOWS_FORCE_STDERR_SPEC",
-    default=None,
-    help="""\
-Force standard error of the program to go to this location. Useful for programs with
-disabled console and programs using the Windows Services Plugin of Nuitka. Defaults
-to not active, use e.g. '%PROGRAM%.err.txt', i.e. file near your program.""",
-)
-
-
 parser.add_option_group(windows_group)
 
 macos_group = OptionGroup(parser, "macOS specific controls")
 
 macos_group.add_option(
-    "--macos-onefile-icon",
-    action="append",
-    dest="icon_path",
-    metavar="ICON_PATH",
-    default=[],
-    help="Add executable icon for binary to use. Can be given only one time. Defaults to Python icon if available.",
-)
-
-macos_group.add_option(
-    "--macos-disable-console",
-    action="store_true",
-    dest="disable_console",
-    default=False,
+    "--macos-target-arch",
+    action="store",
+    dest="macos_target_arch",
+    choices=("universal", "arm64", "x86_64"),
+    metavar="MACOS_TARGET_ARCH",
+    default=None,
     help="""\
-When compiling for macOS, disable the console window and create a GUI
-application. Defaults to off.""",
+What architectures is this to supposed to run on. Default and limit
+is what the running Python allows for. Default is "native" which is
+the architecture the Python is run with.""",
 )
 
 macos_group.add_option(
@@ -1187,6 +1293,16 @@ application. Currently experimental and incomplete. Currently this
 is the only way to unlock disabling of console.Defaults to off.""",
 )
 
+macos_group.add_option(
+    "--macos-app-icon",
+    action="append",
+    dest="icon_path",
+    metavar="ICON_PATH",
+    default=[],
+    help="Add icon for the application bundle to use. Can be given only one time. Defaults to Python icon if available.",
+)
+
+
 macos_group.add_option(
     "--macos-signed-app-name",
     action="store",
@@ -1194,9 +1310,9 @@ macos_group.add_option(
     metavar="MACOS_SIGNED_APP_NAME",
     default=None,
     help="""\
-Name of the application to use for macOS signing. Follow com.yourcompany.appname naming
-results for best results, as these have to be globally unique, and will grant protected
-API accesses.""",
+Name of the application to use for macOS signing. Follow "com.YourCompany.AppName"
+naming results for best results, as these have to be globally unique, and will
+potentially grant protected API accesses.""",
 )
 
 macos_group.add_option(
@@ -1210,6 +1326,19 @@ Name of the product to use in macOS bundle information. Defaults to base
 filename of the binary.""",
 )
 
+macos_group.add_option(
+    "--macos-sign-identity",
+    action="store",
+    dest="macos_sign_identity",
+    metavar="MACOS_APP_VERSION",
+    default="-",
+    help="""\
+When signing on macOS, by default an ad-hoc identify will be used, but with this
+option your get to specify another identity to use. The signing of code is now
+mandatory on macOS and cannot be disabled. Default "-" if not given, which means
+ad-hoc.""",
+)
+
 macos_group.add_option(
     "--macos-app-version",
     action="store",
@@ -1217,16 +1346,33 @@ macos_group.add_option(
     metavar="MACOS_APP_VERSION",
     default=None,
     help="""\
-Product version to use in macOS bundle information. Defaults to 1.0 if
+Product version to use in macOS bundle information. Defaults to "1.0" if
 not given.""",
 )
 
+macos_group.add_option(
+    "--macos-app-protected-resource",
+    action="append",
+    dest="macos_protected_resources",
+    metavar="RESOURCE_DESC",
+    default=[],
+    help="""\
+Request access for macOS protected resources, e.g.
+"NSMicrophoneUsageDescription:Microphone access for recording audio."
+requests access to the microphone and provides an informative text for
+the user, why that is needed. Before the colon, is an OS identifier for
+an access right, then the informative text. Legal values can be found on
+https://developer.apple.com/documentation/bundleresources/information_property_list/protected_resources and
+the option can be specified multiple times. Default empty.""",
+)
+
 
 parser.add_option_group(macos_group)
 
 linux_group = OptionGroup(parser, "Linux specific controls")
 
 linux_group.add_option(
+    "--linux-icon",
     "--linux-onefile-icon",
     action="append",
     dest="icon_path",
@@ -1298,15 +1444,13 @@ plugin_group.add_option(
 )
 
 plugin_group.add_option(
-    "--persist-source-changes",
+    "--show-source-changes",
     action="store_true",
-    dest="persist_source_changes",
+    dest="show_source_changes",
     default=False,
     help="""\
-Write source changes to original Python files. Use with care. May need
-permissions, best for use in a virtualenv to debug if plugin code
-changes work with standard Python or to benefit from bloat removal
-even with pure Python. Default False.""",
+Show source changes to original Python file content before compilation. Mostly
+intended for developing plugins. Default False.""",
 )
 
 
@@ -1314,9 +1458,12 @@ def _considerPluginOptions(logger):
     # Cyclic dependency on plugins during parsing of command line.
     from nuitka.plugins.Plugins import (
         addPluginCommandLineOptions,
+        addStandardPluginCommandLineOptions,
         addUserPluginCommandLineOptions,
     )
 
+    addStandardPluginCommandLineOptions(parser=parser)
+
     for arg in sys.argv[1:]:
         if arg.startswith(("--enable-plugin=", "--plugin-enable=")):
             plugin_names = arg[16:]
@@ -1329,7 +1476,6 @@ def _considerPluginOptions(logger):
             addPluginCommandLineOptions(
                 parser=parser,
                 plugin_names=plugin_names.split(","),
-                data_files_tags=data_files_tags,
             )
 
         if arg.startswith("--user-plugin="):
@@ -1340,9 +1486,7 @@ def _considerPluginOptions(logger):
                     % plugin_name.split("=", 1)[0]
                 )
 
-            addUserPluginCommandLineOptions(
-                parser=parser, filename=plugin_name, data_files_tags=data_files_tags
-            )
+            addUserPluginCommandLineOptions(parser=parser, filename=plugin_name)
 
 
 def _expandProjectArg(arg, filename_arg, for_eval):
@@ -1364,10 +1508,11 @@ def _expandProjectArg(arg, filename_arg, for_eval):
     if isLinux():
         dist_info = getLinuxDistribution()
     else:
-        dist_info = "N/A", "0"
+        dist_info = "N/A", "N/A", "0"
 
     values["Linux_Distribution_Name"] = dist_info[0]
-    values["Linux_Distribution_Version"] = dist_info[1]
+    values["Linux_Distribution_Base"] = dist_info[1] or dist_info[0]
+    values["Linux_Distribution_Version"] = dist_info[2]
 
     if getOS() == "Windows":
         values["WindowsRelease"] = getWindowsRelease()
@@ -1378,7 +1523,7 @@ def _expandProjectArg(arg, filename_arg, for_eval):
 
 
 def _getProjectOptions(logger, filename_arg, module_mode):
-    # Complex stuff, pylint: disable=too-many-branches,too-many-locals
+    # Complex stuff, pylint: disable=too-many-branches,too-many-locals,too-many-statements
 
     if os.path.isdir(filename_arg):
         if module_mode:
@@ -1400,8 +1545,8 @@ def _getProjectOptions(logger, filename_arg, module_mode):
 
     cond_level = -1
 
-    for count, line in enumerate(contents_by_line):
-        match = re.match(b"^\\s*#(\\s+)nuitka-project(.*?):(.*)", line)
+    for line_number, line in enumerate(contents_by_line):
+        match = re.match(b"^\\s*#(\\s*)nuitka-project(.*?):(.*)", line)
 
         if match:
             level, command, arg = match.groups()
@@ -1411,13 +1556,15 @@ def _getProjectOptions(logger, filename_arg, module_mode):
             # Check for empty conditional blocks.
             if expect_block and level <= cond_level:
                 sysexit(
-                    count,
-                    "Error, 'nuitka-project-if' is expected to be followed by block start.",
+                    line_number,
+                    "Error, 'nuitka-project-if|else' is expected to be followed by block start.",
                 )
 
             expect_block = False
 
-            if level <= cond_level:
+            if level == cond_level and command == b"-else":
+                execute_block = not execute_block
+            elif level <= cond_level:
                 execute_block = True
 
             if level > cond_level and not execute_block:
@@ -1430,25 +1577,42 @@ def _getProjectOptions(logger, filename_arg, module_mode):
             if command == "-if":
                 if not arg.endswith(":"):
                     sysexit(
-                        count,
+                        line_number,
                         "Error, 'nuitka-project-if' needs to start a block with a colon at line end.",
                     )
 
-                arg = arg[:-1]
+                arg = arg[:-1].strip()
 
                 expanded = _expandProjectArg(arg, filename_arg, for_eval=True)
-                r = eval(  # We allow the user to run any code, pylint: disable=eval-used
-                    expanded
-                )
+
+                with withNoSyntaxWarning():
+                    r = eval(  # We allow the user to run any code, pylint: disable=eval-used
+                        expanded
+                    )
 
                 # Likely mistakes, e.g. with "in" tests.
                 if r is not True and r is not False:
                     sys.exit(
-                        "Error, 'nuitka-project-if' condition %r (%r) does not yield boolean result %r"
+                        "Error, 'nuitka-project-if' condition %r (expanded to %r) does not yield boolean result %r"
                         % (arg, expanded, r)
                     )
 
                 execute_block = r
+                expect_block = True
+                cond_level = level
+            elif command == "-else":
+                if arg:
+                    sysexit(
+                        line_number,
+                        "Error, 'nuitka-project-else' cannot have argument.",
+                    )
+
+                if cond_level != level:
+                    sysexit(
+                        line_number,
+                        "Error, 'nuitka-project-else' not currently allowed after nested nuitka-project-if.",
+                    )
+
                 expect_block = True
                 cond_level = level
             elif command == "":
