@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -117,3 +117,11 @@ def dump(tree):
     value = toString(tree).rstrip()
 
     Tracing.printLine(value)
+
+
+def appendTreeElement(parent, *args, **kwargs):
+    element = Element(*args, **kwargs)
+
+    parent.append(element)
+
+    return element
