@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,7 +24,7 @@ together and cross-module optimizations are the most difficult to tackle.
 import os
 
 from nuitka import Options, Variables
-from nuitka.containers.oset import OrderedSet
+from nuitka.containers.OrderedSets import OrderedSet
 from nuitka.importing.Importing import (
     getModuleNameAndKindFromFilename,
     locateModule,
@@ -35,7 +35,7 @@ from nuitka.optimizations.TraceCollections import TraceCollectionModule
 from nuitka.PythonVersions import python_version
 from nuitka.SourceCodeReferences import fromFilename
 from nuitka.tree.Operations import DetectUsedModules, visitTree
-from nuitka.tree.SourceReading import readSourceCodeFromFilename
+from nuitka.tree.SourceHandling import readSourceCodeFromFilename
 from nuitka.utils.CStrings import encodePythonIdentifierToC
 from nuitka.utils.FileOperations import getFileContentByLine
 from nuitka.utils.ModuleNames import ModuleName
@@ -93,7 +93,7 @@ class PythonModuleBase(NodeBase):
             _package_name, package_filename, finding = locateModule(
                 module_name=package_name,
                 parent_package=None,
-                level=1,
+                level=0,
             )
 
             # If we can't find the package for Python3.3 that is semi-OK, it might be in a
@@ -135,7 +135,7 @@ class PythonModuleBase(NodeBase):
                 package,
                 using_module=self,
                 usage_tag="package",
-                reason="Containing package",
+                reason="Containing package of '%s'." % self.getFullName(),
                 source_ref=self.source_ref,
             )
 
@@ -657,9 +657,6 @@ class CompiledPythonModule(
         return self.locals_scope
 
     def getRuntimePackageValue(self):
-        # Do not want to get asked for now.
-        assert not Options.shallMakeModule()
-
         if self.isCompiledPythonPackage():
             return self.getFullName().asString()
 
@@ -677,9 +674,6 @@ class CompiledPythonModule(
             return None
 
     def getRuntimeNameValue(self):
-        # Do not want to get asked for now.
-        assert not Options.shallMakeModule()
-
         if self.isMainModule() and Options.hasPythonFlagPackageMode():
             return "__main__"
         else:
@@ -916,14 +910,14 @@ class PythonMainModule(CompiledPythonModule):
             )
 
 
-class PythonShlibModule(PythonModuleBase):
-    kind = "PYTHON_SHLIB_MODULE"
+class PythonExtensionModule(PythonModuleBase):
+    kind = "PYTHON_EXTENSION_MODULE"
 
-    __slots__ = ("used_modules",)
+    __slots__ = ("used_modules", "technical")
 
     avoid_duplicates = set()
 
-    def __init__(self, module_name, source_ref):
+    def __init__(self, module_name, technical, source_ref):
         PythonModuleBase.__init__(self, module_name=module_name, source_ref=source_ref)
 
         # That would be a mistake we just made.
@@ -937,6 +931,9 @@ class PythonShlibModule(PythonModuleBase):
         assert self.getFullName() not in self.avoid_duplicates, self.getFullName()
         self.avoid_duplicates.add(self.getFullName())
 
+        # Required to startup
+        self.technical = technical
+
         self.used_modules = None
 
     def finalize(self):
@@ -948,6 +945,10 @@ class PythonShlibModule(PythonModuleBase):
     def startTraversal(self):
         pass
 
+    def isTechnical(self):
+        """Must be present as it's used in CPython library initialization."""
+        return self.technical
+
     def getPyIFilename(self):
         """Get Python type description filename."""
 
