@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -25,11 +25,11 @@ import os
 import sys
 from optparse import OptionParser
 
-from nuitka.freezer.Standalone import (
+from nuitka.freezer.DllDependenciesWin32 import (
     detectBinaryPathDLLsWindowsDependencyWalker,
 )
 from nuitka.Tracing import my_print
-from nuitka.utils.SharedLibraries import getSxsFromDLL, getWindowsDLLVersion
+from nuitka.utils.SharedLibraries import getDLLVersion, getSxsFromDLL
 from nuitka.utils.Timing import TimerReport
 
 
@@ -43,14 +43,14 @@ def main():
 
     for filename in positional_args:
         my_print("Filename: %s" % filename)
-        my_print("Version Information: %s" % getWindowsDLLVersion(filename))
+        my_print("Version Information: %s" % getDLLVersion(filename))
 
         my_print("SXS information (manifests):")
         sxs = getSxsFromDLL(filename=filename, with_data=True)
         if sxs:
             my_print(sxs)
 
-        my_print("DLLs recursively dependended (depends.exe):")
+        my_print("DLLs recursively depended (depends.exe):")
 
         with TimerReport(
             message="Finding dependencies for %s took %%.2f seconds" % filename
