@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -35,9 +35,10 @@ from nuitka.__past__ import subprocess
 from nuitka.PythonVersions import (
     getPartiallySupportedPythonVersions,
     getSupportedPythonVersions,
+    isDebugPython,
 )
 from nuitka.Tracing import OurLogger, my_print
-from nuitka.tree.SourceReading import readSourceCodeFromFilename
+from nuitka.tree.SourceHandling import readSourceCodeFromFilename
 from nuitka.utils.AppDirs import getCacheDir
 from nuitka.utils.Execution import check_output, getNullInput, getNullOutput
 from nuitka.utils.FileOperations import (
@@ -51,11 +52,11 @@ from nuitka.utils.FileOperations import (
     openTextFile,
     removeDirectory,
 )
+from nuitka.utils.InstalledPythons import findInstalledPython
 from nuitka.utils.Jinja2 import getTemplate
 from nuitka.utils.Utils import getOS
 
 from .SearchModes import (
-    SearchModeAll,
     SearchModeByPattern,
     SearchModeCoverage,
     SearchModeImmediate,
@@ -63,6 +64,8 @@ from .SearchModes import (
     SearchModeResume,
 )
 
+# spellchecker: ignore popenargs,pathsep
+
 test_logger = OurLogger("", base_style="blue")
 
 
@@ -72,19 +75,29 @@ def check_result(*popenargs, **kwargs):
 
     process = subprocess.Popen(stdout=subprocess.PIPE, *popenargs, **kwargs)
     _unused_output, _unused_err = process.communicate()
-    retcode = process.poll()
+    ret_value = process.poll()
 
-    if retcode:
+    if ret_value:
         return False
     else:
         return True
 
 
+_start_dir = None
+
+
 def goMainDir():
+    global _start_dir  # singleton, pylint: disable=global-statement
+    _start_dir = os.getcwd()
+
     # Go its own directory, to have it easy with path knowledge.
     os.chdir(os.path.dirname(os.path.abspath(sys.modules["__main__"].__file__)))
 
 
+def getStartDir():
+    return _start_dir
+
+
 _python_version_str = None
 _python_version = None
 _python_arch = None
@@ -92,6 +105,48 @@ _python_executable = None
 _python_vendor = None
 
 
+def _parsePythonVersionOutput(python_binary):
+    version_output = check_output(
+        (
+            python_binary,
+            "-c",
+            """\
+import sys, os;\
+print(".".join(str(s) for s in list(sys.version_info)[:3]));\
+print(("x86_64" if "AMD64" in sys.version else "x86") if os.name == "nt" else os.uname()[4]);\
+print(sys.executable);\
+print("Anaconda" if os.path.exists(os.path.join(sys.prefix, 'conda-meta')) else "Unknown")\
+""",
+        ),
+        stderr=subprocess.STDOUT,
+    )
+
+    python_version_str = version_output.split(b"\n")[0].strip()
+    python_arch = version_output.split(b"\n")[1].strip()
+    python_executable = version_output.split(b"\n")[2].strip()
+    python_vendor = version_output.split(b"\n")[3].strip()
+
+    if str is not bytes:
+        python_version_str = python_version_str.decode("utf8")
+        python_arch = python_arch.decode("utf8")
+        python_executable = python_executable.decode("utf8")
+        python_vendor = python_vendor.decode("utf8")
+
+    assert type(python_version_str) is str, repr(python_version_str)
+    assert type(python_arch) is str, repr(python_arch)
+    assert type(python_executable) is str, repr(_python_executable)
+
+    python_version = tuple(int(d) for d in python_version_str.split("."))
+
+    return (
+        python_version,
+        python_version_str,
+        python_arch,
+        python_executable,
+        python_vendor,
+    )
+
+
 def setup(suite="", needs_io_encoding=False, silent=False, go_main=True):
     if go_main:
         goMainDir()
@@ -117,37 +172,15 @@ def setup(suite="", needs_io_encoding=False, silent=False, go_main=True):
     if needs_io_encoding and "PYTHONIOENCODING" not in os.environ:
         os.environ["PYTHONIOENCODING"] = "utf-8"
 
-    version_output = check_output(
-        (
-            os.environ["PYTHON"],
-            "-c",
-            """\
-import sys, os;\
-print(".".join(str(s) for s in list(sys.version_info)[:3]));\
-print(("x86_64" if "AMD64" in sys.version else "x86") if os.name == "nt" else os.uname()[4]);\
-print(sys.executable);\
-print("Anaconda" if os.path.exists(os.path.join(sys.prefix, 'conda-meta')) else "Unknown")\
-""",
-        ),
-        stderr=subprocess.STDOUT,
-    )
-
     global _python_version_str, _python_version, _python_arch, _python_executable, _python_vendor  # singleton, pylint: disable=global-statement
 
-    _python_version_str = version_output.split(b"\n")[0].strip()
-    _python_arch = version_output.split(b"\n")[1].strip()
-    _python_executable = version_output.split(b"\n")[2].strip()
-    _python_vendor = version_output.split(b"\n")[3].strip()
-
-    if str is not bytes:
-        _python_version_str = _python_version_str.decode("utf8")
-        _python_arch = _python_arch.decode("utf8")
-        _python_executable = _python_executable.decode("utf8")
-        _python_vendor = _python_vendor.decode("utf8")
-
-    assert type(_python_version_str) is str, repr(_python_version_str)
-    assert type(_python_arch) is str, repr(_python_arch)
-    assert type(_python_executable) is str, repr(_python_executable)
+    (
+        _python_version,
+        _python_version_str,
+        _python_arch,
+        _python_executable,
+        _python_vendor,
+    ) = _parsePythonVersionOutput(python_binary=os.environ["PYTHON"])
 
     if not silent:
         my_print("Using concrete python", _python_version_str, "on", _python_arch)
@@ -157,8 +190,6 @@ print("Anaconda" if os.path.exists(os.path.join(sys.prefix, 'conda-meta')) else
             os.path.dirname(__file__), "..", "..", "..", ".coverage"
         )
 
-    _python_version = tuple(int(d) for d in _python_version_str.split("."))
-
     return _python_version
 
 
@@ -260,11 +291,11 @@ def decideFilenameVersionSkip(filename):
     This codifies certain rules that files can have as suffixes or prefixes
     to make them be part of the set of tests executed for a version or not.
 
-    Generally, an ening of "<major><minor>.py" indicates that it must be that
+    Generally, an ending of "<major><minor>.py" indicates that it must be that
     Python version or higher. There is no need for ending in "26.py" as this
     is the minimum version anyway.
 
-    The "_2.py" indicates a maxmimum version of 2.7, i.e. not Python 3.x, for
+    The "_2.py" indicates a maximum version of 2.7, i.e. not Python 3.x, for
     language syntax no more supported.
     """
 
@@ -458,28 +489,37 @@ def checkSucceedsWithCPython(filename):
     return result == 0
 
 
-def hasDebugPython():
+def getDebugPython():
+    # For all Python, if it's the one also executing the runner, which is
+    # very probably the case, we check that. We don't check the provided
+    # binary here, this could be done as well.
+    if sys.executable == os.environ["PYTHON"] and isDebugPython():
+        return sys.executable
+
     # On Debian systems, these work.
     debug_python = os.path.join("/usr/bin/", os.environ["PYTHON"] + "-dbg")
     if os.path.exists(debug_python):
-        return True
+        return debug_python
 
-    # On Windows systems, these work.
-    debug_python = os.environ["PYTHON"]
-    if debug_python.lower().endswith(".exe"):
-        debug_python = debug_python[:-4]
-    debug_python = debug_python + "_d.exe"
-    if os.path.exists(debug_python):
-        return True
+    # On Fedora systems, these work, but on for Python3
+    debug_python = os.path.join("/usr/bin/", os.environ["PYTHON"] + "-debug")
+    if os.path.exists(debug_python) and _parsePythonVersionOutput(debug_python)[0] >= (
+        3,
+    ):
+        return debug_python
 
-    # For other Python, if it's the one also executing the runner, which is
-    # very probably the case, we check that. We don't check the provided
-    # binary here, this could be done as well.
-    if sys.executable == os.environ["PYTHON"] and hasattr(sys, "gettotalrefcount"):
-        return True
+    # On Windows systems, these work. TODO: Python asserts in Nuitka with
+    # these, not sure why, pylint: disable=using-constant-test
+    if False:
+        debug_python = os.environ["PYTHON"]
+        if debug_python.lower().endswith(".exe"):
+            debug_python = debug_python[:-4]
+        debug_python = debug_python + "_d.exe"
+        if os.path.exists(debug_python):
+            return debug_python
 
     # Otherwise no.
-    return False
+    return None
 
 
 def displayRuntimeTraces(logger, path):
@@ -590,7 +630,7 @@ def reenablePrinting():
         orig_print = None
 
 
-_debug_python = hasattr(sys, "gettotalrefcount")
+_debug_python = isDebugPython()
 
 
 def getTotalReferenceCount():
@@ -764,16 +804,17 @@ Defaults to off.""",
     mode = positional_args[0] if positional_args else "search"
 
     # Avoid having to use options style.
-    if mode in ("search", "only"):
+    if mode in ("search", "only", "coverage"):
         if len(positional_args) >= 2 and not options.pattern:
             options.pattern = positional_args[1]
 
     if mode == "search":
         if options.all:
-            return SearchModeAll()
+            return SearchModeByPattern(start_at=None)
         elif options.pattern:
-            pattern = options.pattern.replace("/", os.path.sep)
-            return SearchModeByPattern(pattern)
+            return SearchModeByPattern(
+                start_at=options.pattern.replace("/", os.path.sep)
+            )
         else:
             return SearchModeImmediate()
     elif mode == "resume":
@@ -785,7 +826,11 @@ Defaults to off.""",
         else:
             assert False
     elif mode == "coverage":
-        return SearchModeCoverage()
+        return SearchModeCoverage(
+            start_at=options.pattern.replace("/", os.path.sep)
+            if options.pattern
+            else None
+        )
     else:
         test_logger.sysexit("Error, using unknown search mode %r" % mode)
 
@@ -797,7 +842,9 @@ def reportSkip(reason, dirname, filename):
     my_print("Skipped, %s (%s)." % (case, reason))
 
 
-def executeReferenceChecked(prefix, names, tests_skipped, tests_stderr, explain=False):
+def executeReferenceChecked(
+    prefix, names, tests_skipped=(), tests_stderr=(), explain=False
+):
     gc.disable()
 
     extract_number = lambda name: int(name.replace(prefix, ""))
@@ -820,7 +867,7 @@ def executeReferenceChecked(prefix, names, tests_skipped, tests_stderr, explain=
             my_print(name + ": SKIPPED (%s)" % tests_skipped[number])
             continue
 
-        # Avoid unraisable output.
+        # Avoid non-raisable output.
         try:
             if number in tests_stderr:
                 sys.stderr = getNullOutput()
@@ -1176,7 +1223,10 @@ def scanDirectoryForTestCases(dirname, template_context=None):
             assert template_context is not None
 
             template = getTemplate(
-                package_name=None, template_name=filename, template_subdir=dirname
+                package_name=None,
+                template_name=filename,
+                template_subdir=dirname,
+                extensions=("jinja2.ext.do",),
             )
 
             code = template.render(name=template.name, **template_context)
@@ -1196,6 +1246,9 @@ def scanDirectoryForTestCaseFolders(dirname):
     filenames = os.listdir(dirname)
 
     for filename in sorted(filenames):
+        if not decideFilenameVersionSkip(filename + ".py"):
+            continue
+
         filename = os.path.join(dirname, filename)
         filename = os.path.relpath(filename)
 
@@ -1371,7 +1424,7 @@ def checkLoadedFileAccesses(loaded_filenames, current_dir):
             if r"windows\winsxs" in loaded_filename:
                 continue
 
-            # Github actions have these in PATH overriding SYSTEMROOT
+            # GitHub actions have these in PATH overriding SYSTEMROOT
             if r"windows performance toolkit" in loaded_filename:
                 continue
             if r"powershell" in loaded_filename:
@@ -1736,3 +1789,40 @@ def getMainProgramFilename(filename):
 Error, no file ends with 'Main.py' or 'Main' in '%s', incomplete test case."""
         % (filename)
     )
+
+
+def getInstalledPythonVersion(python_version, must_exist):
+    result = findInstalledPython(
+        python_versions=(python_version,), module_name=None, module_version=None
+    )
+
+    if result is None and must_exist:
+        test_logger.sysexit(
+            "Error, cannot find required Python version %s installation."
+            % python_version
+        )
+
+    return result
+
+
+_sys_path_path = None
+
+
+def getPythonSysPath():
+    global _sys_path_path  # singleton, pylint: disable=global-statement
+
+    if _sys_path_path is None:
+        _sys_path_path = check_output(
+            [
+                os.environ["PYTHON"],
+                "-c",
+                "import sys, os; print(os.pathsep.join(sys.path))",
+            ]
+        )
+
+        if str is not bytes:
+            _sys_path_path = _sys_path_path.decode("utf8")
+
+        _sys_path_path = _sys_path_path.strip()
+
+    return _sys_path_path
