@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -18,13 +18,14 @@
 """ Utils for file and directory operations.
 
 This provides enhanced and more error resilient forms of standard
-stuff. It will also frequently add sorting for determism.
+stuff. It will also frequently add sorting for determinism.
 
 """
 
 from __future__ import print_function
 
 import errno
+import fnmatch
 import glob
 import os
 import shutil
@@ -36,13 +37,14 @@ from contextlib import contextmanager
 from nuitka.__past__ import (  # pylint: disable=I0021,redefined-builtin
     WindowsError,
     basestring,
+    raw_input,
 )
 from nuitka.PythonVersions import python_version
-from nuitka.Tracing import my_print, options_logger
+from nuitka.Tracing import general, my_print, options_logger
 
 from .Importing import importFromInlineCopy
 from .ThreadedExecutor import RLock, getThreadIdent
-from .Utils import getOS, isWin32Windows
+from .Utils import getOS, isMacOS, isWin32Windows
 
 # Locking seems to be only required for Windows mostly, but we can keep
 # it for all.
@@ -165,25 +167,65 @@ def makePath(path):
             os.makedirs(path)
 
 
+def makeContainingPath(filename):
+    target_dir = os.path.dirname(filename)
+
+    if not os.path.isdir(target_dir):
+        makePath(target_dir)
+
+
 def isPathExecutable(path):
     """Is the given path executable."""
     return os.path.isfile(path) and os.access(path, os.X_OK)
 
 
+# Make sure we don't repeat this too much.
+_real_path_windows_cache = {}
+_powershell_path = None
+
+
 def _getRealPathWindows(path):
-    # Slow, because we are using an external process, we it's only for standalone and Python2,
-    # which is slow already.
-    import subprocess
+    # Slow on Python2, because we are using an external process.
+    # Singleton, pylint: disable=global-statement
+    global _powershell_path
+    if _powershell_path is None:
+        from .Execution import getExecutablePath
 
-    result = subprocess.check_output(
-        """powershell -NoProfile "Get-Item '%s' | Select-Object -ExpandProperty Target" """
-        % path
-    )
+        _powershell_path = getExecutablePath("powershell")
 
-    if str is not bytes:
-        result = result.decode("utf8")
+        # Try to find it only once, otherwise ignore its absence, symlinks are not
+        # that important.
+        if _powershell_path is None:
+            _powershell_path = False
 
-    return os.path.join(os.path.dirname(path), result.rstrip("\r\n"))
+    if path not in _real_path_windows_cache:
+        if _powershell_path:
+            from .Execution import check_output
+
+            result = check_output(
+                [
+                    _powershell_path,
+                    "-NoProfile",
+                    "Get-Item",
+                    path,
+                    "|",
+                    "Select-Object",
+                    "-ExpandProperty",
+                    "Target",
+                ],
+                shell=False,
+            )
+
+            if str is not bytes:
+                result = result.decode("utf8")
+
+            _real_path_windows_cache[path] = os.path.join(
+                os.path.dirname(path), result.rstrip("\r\n")
+            )
+        else:
+            _real_path_windows_cache[path] = path
+
+    return _real_path_windows_cache[path]
 
 
 def getDirectoryRealPath(path):
@@ -202,7 +244,7 @@ def getDirectoryRealPath(path):
     path = os.path.realpath(path)
 
     # Attempt to resolve Windows symlinks on Python2
-    if os.name == "nt" and not os.path.isdir(path):
+    if os.name == "nt" and not os.path.isdir(path) and os.path.exists(path):
         path = _getRealPathWindows(path)
 
     return path
@@ -216,7 +258,7 @@ def listDir(path):
 
     Returns:
         Sorted list of tuples of full filename, and basename of
-        a directory.
+        files in that directory.
 
     Notes:
         Typically the full name and the basename are both needed
@@ -307,6 +349,7 @@ def getSubDirectories(path, ignore_dirs=()):
 
     Args:
         path: directory to create a recursive listing from
+        ignore_dirs: directories named that like will be ignored
 
     Returns:
         Sorted list of all directories below that directory,
@@ -337,6 +380,78 @@ def getSubDirectories(path, ignore_dirs=()):
     return result
 
 
+def listDllFilesFromDirectory(path, prefix=""):
+    """Give a sorted listing of DLLs filenames in a path.
+
+    Args:
+        path: directory to create a DLL listing from
+        prefix: shell pattern to match filename start against, can be None
+
+    Returns:
+        Sorted list of tuples of full filename, and basename of
+        DLLs in that directory.
+
+    Notes:
+        Typically the full name and the basename are both needed
+        so this function simply does both, for ease of use on the
+        calling side.
+    """
+
+    # Accept None value as well.
+    prefix = prefix or ""
+
+    pattern_list = [prefix + "*." + suffix for suffix in ("dll", "so.*", "so", "dylib")]
+
+    for fullpath, filename in listDir(path):
+        for pattern in pattern_list:
+            if fnmatch.fnmatch(filename, pattern):
+                yield fullpath, filename
+                break
+
+
+def getSubDirectoriesWithDlls(path):
+    """Get all directories below a given path.
+
+    Args:
+        path: directory to create a recursive listing from
+
+    Returns:
+        Sorted tuple of all directories below that directory,
+        relative to it, that contain DLL files.
+
+    Notes:
+        This function descends into directories, but does
+        not follow symlinks.
+    """
+
+    result = set()
+
+    for dll_sub_directory in _getSubDirectoriesWithDlls(path):
+        result.add(dll_sub_directory)
+
+    return tuple(sorted(result))
+
+
+def _getSubDirectoriesWithDlls(path):
+    for sub_directory in getSubDirectories(path=path, ignore_dirs=("__pycache__",)):
+        if any(listDllFilesFromDirectory(sub_directory)) or _isMacOSFramework(
+            sub_directory
+        ):
+            yield sub_directory
+
+            candidate = os.path.dirname(sub_directory)
+
+            # Should be string identical, no normalization in is done in "getSubDirectories"
+            while candidate != path:
+                yield candidate
+                candidate = os.path.dirname(candidate)
+
+
+def _isMacOSFramework(path):
+    """Decide if a folder is a framework folder."""
+    return isMacOS() and os.path.isdir(path) and path.endswith(".framework")
+
+
 def deleteFile(path, must_exist):
     """Delete a file, potentially making sure it exists.
 
@@ -400,7 +515,7 @@ def hasFilenameExtension(path, extensions):
 def removeDirectory(path, ignore_errors):
     """Remove a directory recursively.
 
-    On Windows, it happens that operations fail, and succeed when reried,
+    On Windows, it happens that operations fail, and succeed when retried,
     so added a retry and small delay, then another retry. Should make it
     much more stable during tests.
 
@@ -428,6 +543,11 @@ def removeDirectory(path, ignore_errors):
                     raise
 
 
+def resetDirectory(path, ignore_errors):
+    removeDirectory(path=path, ignore_errors=ignore_errors)
+    makePath(path)
+
+
 @contextmanager
 def withTemporaryFile(suffix="", mode="w", delete=True, temp_path=None):
     with tempfile.NamedTemporaryFile(
@@ -517,16 +637,29 @@ def putTextFileContents(filename, contents, encoding=None):
 
 
 @contextmanager
-def withPreserveFileMode(filename):
-    old_mode = os.stat(filename).st_mode
+def withPreserveFileMode(filenames):
+    if type(filenames) is str:
+        filenames = [filenames]
+
+    old_modes = {}
+    for filename in filenames:
+        old_modes[filename] = os.stat(filename).st_mode
+
     yield
-    os.chmod(filename, old_mode)
+
+    for filename in filenames:
+        os.chmod(filename, old_modes[filename])
 
 
 @contextmanager
-def withMadeWritableFileMode(filename):
-    with withPreserveFileMode(filename):
-        os.chmod(filename, int("644", 8))
+def withMadeWritableFileMode(filenames):
+    if type(filenames) is str:
+        filenames = [filenames]
+
+    with withPreserveFileMode(filenames):
+        for filename in filenames:
+            os.chmod(filename, int("644", 8))
+
         yield
 
 
@@ -558,7 +691,7 @@ def renameFile(source_filename, dest_filename):
     try:
         os.rename(source_filename, dest_filename)
     except OSError:
-        shutil.copyfile(source_filename, dest_filename)
+        copyFile(source_filename, dest_filename)
         os.unlink(source_filename)
 
     os.chmod(dest_filename, old_stat.st_mode)
@@ -573,15 +706,15 @@ def copyTree(source_path, dest_path):
 
     Notes:
         This must be used over `shutil.copytree` which has troubles
-        with existing directories.
+        with existing directories on some Python versions.
     """
     if python_version >= 0x380:
         # Python 3.8+ has dirs_exist_ok
         return shutil.copytree(source_path, dest_path, dirs_exist_ok=True)
-    else:
-        from distutils.dir_util import copy_tree
 
-        return copy_tree(source_path, dest_path)
+    from distutils.dir_util import copy_tree
+
+    return copy_tree(source_path, dest_path)
 
 
 def copyFileWithPermissions(source_path, dest_path):
@@ -602,6 +735,35 @@ def copyFileWithPermissions(source_path, dest_path):
         os.chmod(dest_path, source_mode)
 
 
+def copyFile(source_path, dest_path):
+    """Improved version of shutil.copy
+
+    This handles errors with a chance to correct them, e.g. on Windows, files might be
+    locked by running program or virus checkers.
+    """
+
+    while 1:
+        try:
+            shutil.copyfile(source_path, dest_path)
+        except PermissionError as e:
+            if e.errno != errno.EACCES:
+                raise
+
+            general.warning("Problem copying file %s:" % e)
+
+            try:
+                reply = raw_input("Retry? (YES/no) ") or "yes"
+            except EOFError:
+                reply = "no"
+
+            if reply.upper() == "YES":
+                continue
+
+            raise
+
+        break
+
+
 def getWindowsDrive(path):
     """Windows drive for a given path."""
 
@@ -610,7 +772,19 @@ def getWindowsDrive(path):
 
 
 def isPathBelow(path, filename):
-    """Is a path inside of a given directory path."""
+    """Is a path inside of a given directory path
+
+    Args:
+        path: location to be below
+        filename: candidate being checked
+    """
+    if type(path) in (tuple, list):
+        for p in path:
+            if isPathBelow(path=p, filename=filename):
+                return True
+
+        return False
+
     path = os.path.abspath(path)
     filename = os.path.abspath(filename)
 
@@ -656,6 +830,10 @@ def getWindowsShortPathName(filename):
         if needed == 0:
             # Windows only code, pylint: disable=I0021,undefined-variable
 
+            # Permission denied.
+            if ctypes.GetLastError() == 5:
+                return filename
+
             raise WindowsError(
                 ctypes.GetLastError(), ctypes.FormatError(ctypes.GetLastError())
             )
@@ -769,6 +947,8 @@ def resolveShellPatternToFilenames(pattern):
 def withDirectoryChange(path, allow_none=False):
     """Change current directory temporarily in a context."""
 
+    # spellchecker: ignore chdir
+
     if path is not None or not allow_none:
         old_cwd = os.getcwd()
         os.chdir(path)
