@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python test originally created or extracted from other peoples work. The
 #     parts from me are licensed as below. It is at least Free Software where
@@ -57,7 +57,7 @@ from nuitka.tools.testing.Common import (
     test_logger,
 )
 from nuitka.utils.Timing import TimerReport
-from nuitka.utils.Utils import getOS
+from nuitka.utils.Utils import isLinux, isMacOS
 
 
 def displayError(dirname, filename):
@@ -72,7 +72,7 @@ def main():
 
     search_mode = createSearchMode()
 
-    if getOS() == "Linux":
+    if isLinux():
         addExtendedExtraOptions(
             "--linux-onefile-icon=../../doc/Logo/Nuitka-Logo-Symbol.svg"
         )
@@ -99,12 +99,14 @@ def main():
             "cpython_cache",
             # To understand what is slow.
             "timing",
+            # The onefile can warn about zstandard not being installed.
+            "ignore_warnings",
         ]
 
-        if filename == "KeyboardInteruptTest.py":
-            if getOS() == "Darwin":
+        if filename == "KeyboardInterruptTest.py":
+            if isMacOS():
                 reportSkip(
-                    "Exit code from KeybaordInterrupt on macOS is not yet good.",
+                    "Exit code from KeyboardInterrupt on macOS is not yet good.",
                     ".",
                     filename,
                 )
@@ -150,7 +152,7 @@ def main():
 
         binary_filename = filename[:-3] + (".exe" if os.name == "nt" else ".bin")
 
-        if filename == "KeyboardInteruptTest.py":
+        if filename == "KeyboardInterruptTest.py":
             continue
 
         # Then use "strace" on the result.
