@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python test originally created or extracted from other peoples work. The
 #     parts from me are licensed as below. It is at least Free Software where
@@ -26,15 +26,18 @@ from __future__ import print_function
 
 # Make sure, the usual bad ones are not included with anti-bloat.
 
-# nuitka-project: --enable-plugin=anti-bloat
-# nuitka-project: --noinclude-setuptools-mode=error
-# nuitka-project: --noinclude-pytest-mode=error
+# nuitka-project: --noinclude-default-mode=error
 # nuitka-project: --noinclude-custom-mode=numpy.distutils:error
 
-import numpy as np
+# nuitka-project: --noinclude-custom-mode=pydoc:error
 
-a = np.arange(15).reshape(3, 5)
+# To trigger DLL usage on non-Linux.
+import numpy.core.multiarray
+
+a = numpy.arange(15).reshape(3, 5)
 
 print("An array", a)
 
+import numpy.random._bounded_integers
+
 print("OK.")
\ No newline at end of file
